import colorsys
import contextlib
import functools
import hashlib
import os
import pdb
import random
import warnings

from .common_words import common_words, common_words_source


_skip = ["breakword", "contextlib"]
_custom_pdb = pdb.Pdb(skip=_skip)


def _breakpoint():
    if "PYTEST_CURRENT_TEST" in os.environ:
        import pytest

        pytest.set_trace(skip=_skip)

    else:
        _custom_pdb.set_trace()


_acquired = {}
_default_log = print


def set_default_logger(logger):
    """Set the default logger used by log."""
    global _default_log
    _default_log = logger


def get_logger(logger=None):
    """Return the logger, or the default logger if None."""
    return logger or _default_log


def file_source(filename, exclude=set()):
    """Return a list of words picked from the given file.

    One word per line. Words in the exclude set are removed from the list.
    """

    def make():
        if filename not in _acquired:
            if not os.path.exists(filename):
                return []
            words = {word.lower() for word in open(filename).read().split("\n")}
            words -= set(exclude)
            _acquired[filename] = list(sorted(words))
        return list(_acquired[filename])

    return make


class WordGroup:
    """Deterministic word generator.

    Generates a sequence of words from common_words, and then from
    /usr/share/dict/words, if it exists. The sequence deterministically depends
    on the name of the group.
    """

    def __init__(self, name, sources):
        self.name = name
        self.hash = int(hashlib.md5(self.name.encode()).hexdigest(), base=16)
        self.words = []
        self.sources = list(reversed(sources))
        self.random = random.Random(self.hash)
        self.index = 0
        self.current = None

    def gen(self):
        """Generate the next word."""
        if not self.words:
            if not self.sources:
                self.words = []
            else:
                next_source = self.sources.pop()
                self.words = next_source()
                self.random.shuffle(self.words)
        if self.words:
            self.current = self.words.pop()
        else:
            self.index += 1
            self.current = str(self.index)
        return self.current

    def after(self, **kw):
        after(group=self, **kw)

    def log(self, *objs, **kw):
        log(*objs, group=self, **kw)

    def word(self, **kw):
        word(group=self, **kw)

    def brk(self, **kw):
        brk(group=self, **kw)

    def logbrk(self, *objs, **kw):
        logbrk(*objs, group=self, **kw)

    def wordbrk(self, **kw):
        wordbrk(group=self, **kw)

    def term_color(self):
        """Generate a terminal color for this group."""
        return 30 + (self.hash % 8)

    def rgb(self):
        """Generate an RGB color string for this group."""
        # We generate the color in the YIQ space first because the Y component,
        # corresponding to brightness, is fairly accurate, so we can easily
        # restrict it to a range that looks decent on a white background, and
        # then convert to RGB with the standard colorsys package. The IQ
        # components control hue and have bizarre valid ranges.
        h = self.hash
        # 0.3 <= Y <= 0.6
        y = 0.3 + ((h & 0xFF) / 0xFF) * 0.4
        h >>= 16
        i = (((h & 0xFF) - 0x80) / 0x80) * 0.5957
        h >>= 16
        q = (((h & 0xFF) - 0x80) / 0x80) * 0.5226
        r, g, b = colorsys.yiq_to_rgb(y, i, q)
        r = int(r * 255)
        g = int(g * 255)
        b = int(b * 255)
        return f"rgb({r}, {g}, {b})"


class GroupsDict(dict):
    """Dictionary from str to WordGroup.

    Keys can be accessed using attribute syntax, e.g. groups.xyz instead of
    groups["xyz"].
    """

    def __getattr__(self, attr):
        return self[attr]

    def __missing__(self, group):
        self[group] = grp = WordGroup(
            name=group,
            sources=[
                common_words_source,
                file_source("/usr/share/dict/words", exclude=common_words),
            ],
        )
        return grp


groups = GroupsDict()


class BreakwordState:
    """State of the breakword generator after generation.

    Attributes:
        group: The WordGroup in use.
        word: The last word generated by the group.
        match: Whether the word matches the word we are watching for.
        logger: Logging function to use (e.g. print).
    """

    def __init__(self, group, word, match, logger=None):
        self.group = group
        self.word = word
        self.match = match
        self._logger = logger

    @property
    def logger(self):
        return get_logger(self._logger)

    def breakpoint(self, force=False):
        """Set a breakpoint if self.match or force is True."""
        if force or self.match:
            _breakpoint()

    def log(self, *objs, force=False):
        """Log the given objects if self.match or force is True."""
        if force or self.match:
            self.logger(self, *objs)

    def __bool__(self):
        """Return whether the state matches the watched word."""
        return self.match

    def __str__(self):
        gname = f"{self.group.name}:" if self.group.name else ""
        color = self.group.term_color()
        return f"\033[1;{color};40m⏎ {gname}{self.word}\033[0m"


def _get_watch(group, watch):
    def parse(w):
        if w is not None and ":" in w:
            g, w = w.split(":")
            g = groups[g]
            if g != group:
                return None
            return w
        else:
            return w

    if watch is None:
        watch = os.environ.get("BREAKWORD")

    if watch is None:
        watch = []

    if isinstance(watch, str):
        watch = watch.split(",")

    results = [w for w in [parse(w) for w in watch] if w is not None]
    return results


def breakword(group=None, gen=True, watch=None, **kwargs):
    """Generate a word in the given group and return a BreakwordState.

    Arguments:
        group: The WordGroup to use to generate the word, or some arbitrary
            string to identify the WordGroup.
        gen: Whether to generate a new word or reuse the last one (defaults
            to True).
        watch: A word or "group:word" expression representing a word to
            watch for. If None, no word is watched. Can also be a list.
    """
    if group is None:
        group = groups[""]

    watch = _get_watch(group, watch)
    word = group.gen() if gen else group.current
    match = word is not None and word in watch
    return BreakwordState(group, word, match, **kwargs)


def after(watch=None, **kw):
    """Return a state that is truthy when the last generated word == watch."""
    return breakword(watch=watch, gen=False, **kw)


def word(**kw):
    """Generate a word and return it as a string."""
    return breakword(**kw).word


def log(*objs, **kw):
    """Generate a word and log it alongside a set of objects."""
    breakword(**kw).log(*objs, force=True)


def brk(**kw):
    """Set a breakpoint when the last word generated is the one watched for."""
    breakword(gen=False, **kw).breakpoint()


def wordbrk(**kw):
    """Call word() then brk()."""
    bw = breakword(**kw)
    bw.breakpoint()
    return bw.word


def logbrk(*objs, **kw):
    """Call log() then brk()."""
    bw = breakword(**kw)
    bw.log(*objs, force=True)
    bw.breakpoint()


def breakpoint(*args, **kwargs):
    """Drop-in breakpoint function.

    When no argument is given it works like the normal breakpoint, otherwise
    it works like logbrk.
    """
    if args:
        logbrk(*args, **kwargs)
    else:
        _breakpoint()


def _track(cls, method, field, wordfn):
    m = getattr(cls, method)

    @functools.wraps(m)
    def new_method(self, *args, **kwargs):
        result = m(self, *args, **kwargs)
        setattr(self, field, wordfn())
        return result

    setattr(cls, method, new_method)
    return cls


def track_creation(*classes, function=wordbrk, field="breakword", **kwargs):
    """Track instances of the given classes.

    The __init__ of each class is modified to also set the breakword field
    to a freshly generated word. Use the BREAKWORD environment variable to
    set a breakpoint to where the instance was created.
    """
    for cls in classes:
        _track(
            cls,
            method="__init__",
            field=field,
            wordfn=lambda: function(**kwargs),
        )
    return cls


def track(obj, function=wordbrk, field="breakword", all=False, **kwargs):
    """Return the object, but track it with the `breakword` attribute."""
    word = function(**kwargs)
    try:
        if all:
            li = getattr(obj, field, [])
            if isinstance(li, list):
                li.append(word)
            else:
                warnings.warn(
                    f"breakword.track was unable to accumulate words into object.{field} because that field is already used"
                )
            setattr(obj, field, li)
        else:
            setattr(obj, field, word)
    except AttributeError:
        warnings.warn(
            f"breakword.track cannot track object of type {type(obj)}"
        )
    return obj



"""List of the 1000 most common English words.

I may have edited it a little :)
"""


def common_words_source():
    return list(common_words)


common_words = [
    "ability",
    "able",
    "about",
    "above",
    "accept",
    "according",
    "account",
    "across",
    "act",
    "action",
    "activity",
    "actually",
    "add",
    "address",
    "administration",
    "admit",
    "adult",
    "affect",
    "after",
    "again",
    "against",
    "age",
    "agency",
    "agent",
    "ago",
    "agree",
    "agreement",
    "ahead",
    "air",
    "all",
    "allow",
    "almost",
    "alone",
    "along",
    "already",
    "also",
    "although",
    "always",
    "among",
    "amount",
    "analysis",
    "and",
    "animal",
    "another",
    "answer",
    "any",
    "anyone",
    "anything",
    "appear",
    "apply",
    "approach",
    "area",
    "argue",
    "arm",
    "around",
    "arrive",
    "art",
    "article",
    "artist",
    "as",
    "ask",
    "assume",
    "at",
    "attack",
    "attention",
    "attorney",
    "audience",
    "author",
    "authority",
    "auto",
    "available",
    "avoid",
    "away",
    "baby",
    "back",
    "bad",
    "bag",
    "ball",
    "banana",
    "bank",
    "bar",
    "base",
    "be",
    "beat",
    "beautiful",
    "because",
    "become",
    "bed",
    "before",
    "begin",
    "behavior",
    "behind",
    "believe",
    "benefit",
    "best",
    "better",
    "between",
    "beyond",
    "big",
    "bill",
    "billion",
    "bit",
    "black",
    "blood",
    "blue",
    "board",
    "body",
    "book",
    "born",
    "both",
    "box",
    "boy",
    "break",
    "bring",
    "brother",
    "budget",
    "build",
    "building",
    "business",
    "but",
    "buy",
    "by",
    "call",
    "camera",
    "campaign",
    "can",
    "canada",
    "cancer",
    "candidate",
    "capital",
    "car",
    "card",
    "care",
    "career",
    "carry",
    "case",
    "catch",
    "cause",
    "cell",
    "center",
    "central",
    "century",
    "certain",
    "certainly",
    "chair",
    "challenge",
    "chance",
    "change",
    "character",
    "charge",
    "check",
    "cherry",
    "child",
    "choice",
    "choose",
    "church",
    "citizen",
    "city",
    "civil",
    "claim",
    "class",
    "clear",
    "clearly",
    "close",
    "coach",
    "cold",
    "collection",
    "college",
    "color",
    "come",
    "commercial",
    "common",
    "community",
    "company",
    "compare",
    "computer",
    "concern",
    "condition",
    "conference",
    "congress",
    "consider",
    "consumer",
    "contain",
    "continue",
    "control",
    "cost",
    "could",
    "country",
    "couple",
    "course",
    "court",
    "cover",
    "create",
    "crime",
    "cultural",
    "culture",
    "cup",
    "current",
    "customer",
    "cut",
    "dark",
    "data",
    "daughter",
    "day",
    "dead",
    "deal",
    "death",
    "debate",
    "decade",
    "decide",
    "decision",
    "deep",
    "defense",
    "degree",
    "democratic",
    "describe",
    "design",
    "despite",
    "detail",
    "determine",
    "develop",
    "development",
    "die",
    "difference",
    "different",
    "difficult",
    "dinner",
    "direction",
    "director",
    "discover",
    "discuss",
    "discussion",
    "disease",
    "do",
    "doctor",
    "dog",
    "door",
    "down",
    "draw",
    "dream",
    "drive",
    "drop",
    "drug",
    "during",
    "each",
    "early",
    "east",
    "easy",
    "eat",
    "economic",
    "economy",
    "edge",
    "education",
    "effect",
    "effort",
    "eight",
    "either",
    "election",
    "else",
    "employee",
    "end",
    "energy",
    "enjoy",
    "enough",
    "enter",
    "entire",
    "environment",
    "environmental",
    "especially",
    "establish",
    "even",
    "evening",
    "event",
    "ever",
    "every",
    "everybody",
    "everyone",
    "everything",
    "evidence",
    "exactly",
    "example",
    "executive",
    "exist",
    "expect",
    "experience",
    "expert",
    "explain",
    "eye",
    "face",
    "fact",
    "factor",
    "fail",
    "fall",
    "family",
    "far",
    "fast",
    "father",
    "fear",
    "federal",
    "feel",
    "feeling",
    "few",
    "field",
    "fight",
    "figure",
    "fill",
    "film",
    "final",
    "finally",
    "financial",
    "find",
    "fine",
    "finger",
    "finish",
    "fire",
    "firm",
    "first",
    "fish",
    "five",
    "floor",
    "fly",
    "focus",
    "follow",
    "food",
    "foot",
    "for",
    "force",
    "foreign",
    "forget",
    "form",
    "former",
    "forward",
    "four",
    "free",
    "friend",
    "from",
    "front",
    "full",
    "fund",
    "future",
    "game",
    "gan",
    "garden",
    "gas",
    "general",
    "generation",
    "get",
    "girl",
    "give",
    "glass",
    "go",
    "goal",
    "good",
    "government",
    "great",
    "green",
    "ground",
    "group",
    "grow",
    "growth",
    "guess",
    "gun",
    "guy",
    "hair",
    "half",
    "hand",
    "hang",
    "happen",
    "happy",
    "hard",
    "have",
    "he",
    "head",
    "health",
    "hear",
    "heart",
    "heat",
    "heavy",
    "help",
    "her",
    "here",
    "herself",
    "high",
    "him",
    "himself",
    "his",
    "history",
    "hit",
    "hold",
    "home",
    "hope",
    "hospital",
    "hot",
    "hotel",
    "hour",
    "house",
    "how",
    "however",
    "huge",
    "human",
    "hundred",
    "husband",
    "idea",
    "identify",
    "if",
    "image",
    "imagine",
    "impact",
    "important",
    "improve",
    "in",
    "include",
    "including",
    "increase",
    "indeed",
    "indicate",
    "individual",
    "industry",
    "information",
    "inside",
    "instead",
    "institution",
    "interest",
    "interesting",
    "international",
    "interview",
    "into",
    "investment",
    "involve",
    "issue",
    "it",
    "item",
    "its",
    "itself",
    "job",
    "join",
    "just",
    "keep",
    "key",
    "kid",
    "kill",
    "kind",
    "kitchen",
    "know",
    "knowledge",
    "land",
    "language",
    "large",
    "last",
    "late",
    "later",
    "laugh",
    "law",
    "lawyer",
    "lay",
    "lead",
    "leader",
    "learn",
    "least",
    "leave",
    "left",
    "leg",
    "legal",
    "less",
    "let",
    "letter",
    "level",
    "lie",
    "life",
    "light",
    "like",
    "likely",
    "line",
    "list",
    "listen",
    "little",
    "live",
    "local",
    "long",
    "look",
    "lose",
    "loss",
    "lot",
    "love",
    "low",
    "machine",
    "magazine",
    "main",
    "maintain",
    "major",
    "majority",
    "make",
    "man",
    "manage",
    "management",
    "manager",
    "many",
    "market",
    "marriage",
    "material",
    "matter",
    "may",
    "maybe",
    "me",
    "mean",
    "measure",
    "media",
    "medical",
    "meet",
    "meeting",
    "member",
    "memory",
    "mention",
    "message",
    "method",
    "middle",
    "might",
    "military",
    "million",
    "mind",
    "minute",
    "miss",
    "mission",
    "model",
    "modern",
    "moment",
    "money",
    "monkey",
    "month",
    "monty",
    "more",
    "morning",
    "most",
    "mother",
    "mouth",
    "move",
    "movement",
    "movie",
    "much",
    "music",
    "must",
    "my",
    "myself",
    "name",
    "nation",
    "national",
    "natural",
    "nature",
    "near",
    "nearly",
    "necessary",
    "need",
    "network",
    "never",
    "new",
    "news",
    "newspaper",
    "next",
    "nice",
    "night",
    "no",
    "none",
    "nor",
    "north",
    "not",
    "note",
    "nothing",
    "notice",
    "now",
    "number",
    "occur",
    "of",
    "off",
    "offer",
    "office",
    "officer",
    "official",
    "often",
    "oh",
    "oil",
    "ok",
    "old",
    "on",
    "once",
    "one",
    "only",
    "onto",
    "open",
    "operation",
    "opportunity",
    "option",
    "or",
    "order",
    "organization",
    "other",
    "others",
    "our",
    "out",
    "outside",
    "over",
    "own",
    "owner",
    "page",
    "pain",
    "painting",
    "paper",
    "parent",
    "part",
    "partner",
    "party",
    "pass",
    "past",
    "patient",
    "pattern",
    "pay",
    "peace",
    "people",
    "per",
    "perform",
    "performance",
    "perhaps",
    "period",
    "person",
    "personal",
    "phone",
    "physical",
    "pick",
    "picture",
    "piece",
    "pizza",
    "place",
    "plan",
    "plant",
    "play",
    "player",
    "point",
    "police",
    "policy",
    "political",
    "politics",
    "poor",
    "popular",
    "population",
    "position",
    "positive",
    "possible",
    "poutine",
    "power",
    "practice",
    "prepare",
    "present",
    "president",
    "pressure",
    "pretty",
    "prevent",
    "price",
    "private",
    "probably",
    "problem",
    "process",
    "produce",
    "product",
    "production",
    "professional",
    "professor",
    "program",
    "project",
    "property",
    "protect",
    "prove",
    "provide",
    "public",
    "pull",
    "purpose",
    "push",
    "put",
    "python",
    "quality",
    "quest",
    "question",
    "quickly",
    "quite",
    "race",
    "radio",
    "raise",
    "range",
    "rate",
    "rather",
    "reach",
    "read",
    "ready",
    "real",
    "reality",
    "realize",
    "really",
    "reason",
    "receive",
    "recent",
    "recently",
    "recognize",
    "record",
    "red",
    "reduce",
    "reflect",
    "region",
    "relate",
    "relationship",
    "religious",
    "remain",
    "remember",
    "remove",
    "report",
    "represent",
    "require",
    "research",
    "resource",
    "respond",
    "response",
    "rest",
    "result",
    "return",
    "reveal",
    "rich",
    "right",
    "rise",
    "risk",
    "road",
    "rock",
    "role",
    "room",
    "rule",
    "run",
    "safe",
    "same",
    "save",
    "say",
    "scene",
    "school",
    "science",
    "scientist",
    "score",
    "sea",
    "season",
    "seat",
    "second",
    "section",
    "security",
    "see",
    "seek",
    "seem",
    "sell",
    "send",
    "senior",
    "sense",
    "series",
    "serious",
    "serve",
    "service",
    "set",
    "seven",
    "several",
    "sex",
    "sexual",
    "shake",
    "share",
    "she",
    "shoot",
    "short",
    "shot",
    "should",
    "shoulder",
    "show",
    "side",
    "sign",
    "significant",
    "similar",
    "simple",
    "simply",
    "since",
    "sing",
    "single",
    "sister",
    "sit",
    "site",
    "situation",
    "six",
    "size",
    "skill",
    "skin",
    "small",
    "smile",
    "so",
    "social",
    "society",
    "soldier",
    "some",
    "somebody",
    "someone",
    "something",
    "sometimes",
    "son",
    "song",
    "soon",
    "sort",
    "sound",
    "source",
    "south",
    "southern",
    "space",
    "speak",
    "special",
    "specific",
    "speech",
    "spend",
    "sport",
    "spring",
    "staff",
    "stage",
    "stand",
    "standard",
    "star",
    "start",
    "state",
    "statement",
    "station",
    "stay",
    "step",
    "still",
    "stock",
    "stop",
    "store",
    "story",
    "strategy",
    "street",
    "strong",
    "structure",
    "student",
    "study",
    "stuff",
    "style",
    "subject",
    "success",
    "successful",
    "such",
    "suddenly",
    "suffer",
    "suggest",
    "summer",
    "support",
    "sure",
    "surface",
    "system",
    "table",
    "take",
    "talk",
    "task",
    "tax",
    "teach",
    "teacher",
    "team",
    "technology",
    "television",
    "tell",
    "ten",
    "tend",
    "term",
    "test",
    "than",
    "thank",
    "that",
    "the",
    "their",
    "them",
    "themselves",
    "then",
    "theory",
    "there",
    "these",
    "they",
    "thing",
    "think",
    "third",
    "this",
    "those",
    "though",
    "thought",
    "thousand",
    "threat",
    "three",
    "through",
    "throughout",
    "throw",
    "thus",
    "time",
    "to",
    "today",
    "together",
    "tonight",
    "too",
    "top",
    "total",
    "tough",
    "toward",
    "town",
    "trade",
    "traditional",
    "training",
    "travel",
    "treat",
    "treatment",
    "tree",
    "trial",
    "trip",
    "trouble",
    "true",
    "truth",
    "try",
    "turn",
    "tv",
    "two",
    "type",
    "under",
    "understand",
    "unit",
    "until",
    "up",
    "upon",
    "us",
    "use",
    "usually",
    "value",
    "various",
    "very",
    "victim",
    "view",
    "violence",
    "visit",
    "voice",
    "vote",
    "wait",
    "walk",
    "wall",
    "want",
    "war",
    "watch",
    "water",
    "way",
    "we",
    "weapon",
    "wear",
    "week",
    "weight",
    "well",
    "west",
    "western",
    "what",
    "whatever",
    "when",
    "where",
    "whether",
    "which",
    "while",
    "white",
    "who",
    "whole",
    "whom",
    "whose",
    "why",
    "wide",
    "wife",
    "will",
    "win",
    "wind",
    "window",
    "wish",
    "with",
    "within",
    "without",
    "woman",
    "wonder",
    "word",
    "work",
    "worker",
    "world",
    "worry",
    "would",
    "write",
    "writer",
    "wrong",
    "yard",
    "yeah",
    "year",
    "yes",
    "yet",
    "you",
    "young",
    "your",
    "yourself",
]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['breakword']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'breakword',
    'version': '0.3.0',
    'description': 'Mixing breakpoints with print debugging.',
    'long_description': '\n# breakword\n\n`breakword` is a small debugging utility that combines print debugging with breakpoint debugging. It aims to facilitate debugging the kind of problem where you might use print statements to quickly spot where something seems to be off, and then switch to a step by step debugger.\n\n`breakword` normally requires running your program twice and will only work properly if it is deterministic.\n\n\n## How to use\n\n1. Set the `PYTHONBREAKPOINT` environment variable to `breakword.breakpoint`.\n\n2. Use `breakpoint` like a `print` statement:\n\n```python\nfor i in range(10):\n    breakpoint(i)\n```\n\nThis will print out something like this:\n\n```\n$ python example.py\n⏎ standard 0\n⏎ sound 1\n⏎ character 2\n⏎ thank 3\n⏎ play 4\n⏎ however 5\n⏎ fish 6\n⏎ cultural 7\n⏎ either 8\n⏎ and 9\n```\n\n3. Use the `BREAKWORD` environment variable to set a breakpoint to what you want to investigate further. For instance, if you want to stop when `i == 6` in the above program, you can run the following command:\n\n\n```\n$ env BREAKWORD=fish python example.py\n⏎ standard 0\n⏎ sound 1\n⏎ character 2\n⏎ thank 3\n⏎ play 4\n⏎ however 5\n⏎ fish 6\n> example.py(2)<module>()\n-> for i in range(10):\n(Pdb) i\n6\n```\n\nYou can also give a comma-separated list of words, e.g. `BREAKWORD=sound,fish`.\n\n**Note:** `breakpoint()` with no arguments retains the normal behavior.\n\n![demo](https://raw.githubusercontent.com/breuleux/breakword/master/media/demo.png)\n\n\n## More functions\n\n* `breakword.log(*things, **config)`: Print a word and optionally other things after it.\n\n* `breakword.brk(watch=None, **config)`: Sets a breakpoint to trigger after `log` printed out the given word. If `watch` is `None` or not given, the `BREAKWORD` environment variable is consulted. If the variable is not set, nothing will happen.\n  * This is equivalent to `breakword.after(word).breakpoint()`.\n\n* `breakword.after(watch=None, **config)`: Returns an object that evaluates to `True` right after `log` printed out the given watch word. As with `brk`, if `watch` is `None` or not given, the `BREAKWORD` environment variable is consulted.\n\n* `breakword.word(**config)`: Returns the next word as a string. You can print it yourself, in which case it\'s basically like `log`, or you can store it in an object.\n\n* `breakword.logbrk(**config)`: Calls `log` and then `brk`.\n\n* `breakword.wordbrk(**config)`: Calls `word` and then `brk`. The word is returned.\n\n* `breakword.set_default_logger(logger)`: Set the logging function to use (defaults to `print`)\n\n\n## Tracking objects\n\n* `breakword.track(obj, all=False)` will set the `breakword` attribute in the object to the next word in the list. By setting the `BREAKWORD` environment variable, you will set a breakpoint to the corresponding call to `track`. Set the `all` argument to `True` and the attribute will contain a list. Note: this will not work if `obj` is an integer or string, in those cases track will print a warning.\n\n* `breakword.track_creation(*classes)` will set the `breakword` attribute on all instances of the given classes, when they are created. That way, you can set a breakpoint back to the creation of some object of interest.\n\n\n## Groups\n\nUse `breakword.groups.<name>` to get a "word group" with the given name. Each group generates words independently and will therefore not interfere with each other. They have `log`, `brk`, `after`, `word`, etc. as methods. The default group is `groups[""]`.\n\n\n```python\nfrom breakword import groups\n\nassert groups.aardvark == groups["aardvark"]\n\n# Log "a" in the aardvark group\ngroups.aardvark.log("a")\n\n# Log "b" in the pelican group\ngroups.pelican.log("b")\n\n# Get the next word in the pelican group\nword = groups.pelican.word()\n\n# Conditional behavior to perform only after the word "cherry"\nif groups.pelican.after("cherry"):\n    print("blah")\n```\n',
    'author': 'Olivier Breuleux',
    'author_email': 'breuleux@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/breuleux/breakword',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
